#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout(std430 ) readonly buffer BStreamInfo
{
	uint StreamInfo[];
} StreamInfo;
layout(std430 ) readonly buffer BGPUSimData
{
	uint GPUSimData[];
} GPUSimData;
layout(std430 ) readonly buffer BEnabledsOffsets
{
	uint EnabledsOffsets[];
} EnabledsOffsets;
layout(std430 ) buffer BOutIndirect
{
	uint OutIndirect[];
} OutIndirect;
layout(std430 ) readonly buffer BMeshIDsOffsets
{
	uint MeshIDsOffsets[];
} MeshIDsOffsets;
layout(std430 ) readonly buffer BLODsOffsets
{
	uint LODsOffsets[];
} LODsOffsets;
layout( std140) uniform ULODConstants
{
	uint PerLODMeshCount[16];
	uint LODMeshOffsets[17];
} LODConstants;

layout( std140) uniform UInfo
{
	uint DrawRequest;
	uint MeshCount;
	uint LODCount;
} Info;



void ComputeMain( in SComputeInput  cInput  )
{
	const uint particleID = cInput.GlobalThreadID.x;
	const uint count =  StreamInfo.StreamInfo [0] ;
	if (particleID >= count)
		return;

	const uint drawRequest =  Info.DrawRequest ;

	const uint meshCount =  Info.MeshCount ;

	const uint enabledsOffset =  EnabledsOffsets.EnabledsOffsets [ drawRequest ] ;
	const uint enabled =  GPUSimData.GPUSimData [enabledsOffset + particleID ] ;
	if (enabled == 0)
		return;

	const uint _LODCount =  Info.LODCount ;
	const uint LODsOffset =  LODsOffsets.LODsOffsets [ drawRequest ] ;
	const uint LOD = min( GPUSimData.GPUSimData [LODsOffset + particleID ] , _LODCount - 1);
	const uint LODLevelToMeshID =  LODConstants.LODMeshOffsets [LOD];

	const uint submeshCount =  LODConstants.PerLODMeshCount [LOD];
	const uint indirectBufferSize = 5;

	const uint meshIDsOffset =  MeshIDsOffsets.MeshIDsOffsets [ drawRequest ] ;
	const uint meshID = min(uint( uintBitsToFloat(GPUSimData.GPUSimData [meshIDsOffset + particleID ]) ), submeshCount - 1);

	uint dummyOutput;

	dummyOutput = atomicAdd( OutIndirect.OutIndirect [ indirectBufferSize * (meshCount * drawRequest + LODLevelToMeshID + meshID) + 1 ], floatBitsToUint(1) ) ;
}


layout(local_size_x = 128, local_size_y = 1, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
