#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout(std430 ) readonly buffer BStreamInfo
{
	uint StreamInfo[];
} StreamInfo;
layout(std430 ) readonly buffer BGPUSimData
{
	uint GPUSimData[];
} GPUSimData;
layout(std430 ) readonly buffer BSelfIDsOffsets
{
	uint SelfIDsOffsets[];
} SelfIDsOffsets;
layout(std430 ) readonly buffer BParentIDsOffsets
{
	uint ParentIDsOffsets[];
} ParentIDsOffsets;
layout(std430 ) readonly buffer BEnabledsOffsets
{
	uint EnabledsOffsets[];
} EnabledsOffsets;
layout(std430 ) buffer BOutSortKeys
{
	uint OutSortKeys[];
} OutSortKeys;
layout(std430 ) buffer BOutIndirection
{
	uint OutIndirection[];
} OutIndirection;
layout(std430 ) buffer BOutIndirectDraw
{
	uint OutIndirectDraw[];
} OutIndirectDraw;

layout( std140) uniform UInfo
{
	uint DrawRequest;
} Info;



void ComputeMain( in SComputeInput  cInput  )
{
	const uint particleID = cInput.GlobalThreadID.x;
	const uint drawRequest =  Info.DrawRequest ;

	const uint selfIDOffset =  SelfIDsOffsets.SelfIDsOffsets [ drawRequest ] ;
	const uint selfID =  uvec2( GPUSimData.GPUSimData [selfIDOffset + particleID * 2 ], GPUSimData.GPUSimData [(selfIDOffset + particleID * 2 ) + 1]) .x;

	const uint parentIDOffset =  ParentIDsOffsets.ParentIDsOffsets [ drawRequest ] ;
	const uint parentID =  uvec2( GPUSimData.GPUSimData [parentIDOffset + particleID * 2 ], GPUSimData.GPUSimData [(parentIDOffset + particleID * 2 ) + 1]) .x;

	const uint enabledOffset =  EnabledsOffsets.EnabledsOffsets [ drawRequest ] ;
	const uint enabled =  GPUSimData.GPUSimData [enabledOffset + particleID ] ;





	uvec2 key;
	key.x = (parentID >> 6) & 0x00000FFF;
	key.y = ((parentID << 26) & 0xFC000000) | ((~selfID) & 0x03FFFFFF);

	if ((particleID >=  StreamInfo.StreamInfo [0] ) || (enabled == 0u))
		key = ~uvec2(0, 0);
	else
	{
		const uint indirectBufferSize = 5;
		uint dummyOutput;
		dummyOutput = atomicAdd( OutIndirectDraw.OutIndirectDraw [ indirectBufferSize * drawRequest + 1 ], floatBitsToUint(1) ) ;
	}

	OutSortKeys.OutSortKeys [ particleID * 2 ] = key.x; OutSortKeys.OutSortKeys [( particleID * 2 ) + 1] = key.y ;

	OutIndirection.OutIndirection [ particleID ] = particleID; ;
}

layout(local_size_x = 128, local_size_y = 1, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
